import logging 

from aiohttp import web, hdrs

from web_interface.auth import check_auth

async def edit_printer(request):
    check_auth(request)
    if request.content_type == 'application/json':
        post_data = await request.json()
    else:
        post_data = await request.post()
    logger = logging.getLogger('edit_printer')
    if not post_data:
        try:
            vid, pid, snr = request.query.get('printer_id').split('_')
            post_data = {'VID': vid, 'PID': pid, 'SNR': snr}

        except (KeyError, ValueError, TypeError):
            raise web.HTTPBadRequest(text='No printer id found in post or query')
    for pi in request.app['3dp_app'].printer_interfaces:
        for key in ('VID', 'PID', 'SNR'):
            logger.info(f"R/P: {pi.usb_info[key]}/{post_data[key]}")
            if pi.usb_info[key] != post_data[key]:
                break
        else:
            if request.method == hdrs.METH_POST:
                result_dict = pi.edit_printer(post_data)
            else:
                result_dict = pi.edit_printer({})
            return web.json_response(data=result_dict)
    raise web.HTTPNotFound(text='No printer with that id')
